@@ -1,4 +1,4 @@
-__version__ = "0.7.4"
+__version__ = "0.7.10"
 
 from ._static_view import jupyter_displayable_output, insight
 from ._utilities import merge_rgb
@@ -16,5 +16,8 @@ from ._picker import picker
 from ._assist import assist
 from ._switch import switch
 from ._colormaps import create_colormap
+from ._imshow import imshow
+from ._animate import animate, animate_curtain
+from ._display_range import display_range
 
 
