@@ -215,11 +215,13 @@ def annotate(
         # connect user interface with event
         slice_slider.observe(update_display)
 
-        return ipywidgets.VBox([
+        result = ipywidgets.VBox([
             ipywidgets.HBox([_no_resize(view), tool_box]),
             slice_slider
         ])
     else:
-        return ipywidgets.VBox([
+        result = ipywidgets.VBox([
             ipywidgets.HBox([_no_resize(view), tool_box]),
         ])
+    result.update = update_display
+    return result
