@@ -35,6 +35,8 @@ def jupyter_displayable_output(
 class StackViewNDArray(np.ndarray):
 
     def __new__(cls, input_array, library_name=None, help_url=None):
+        if 'cupy.ndarray' in str(type(input_array)):
+            input_array = input_array.get()
         obj = np.asarray(input_array).view(cls)
         obj.library_name = library_name
         obj.help_url = help_url
@@ -51,6 +53,8 @@ class StackViewNDArray(np.ndarray):
                 -------
                 HTML text with the image and some properties.
                 """
+        if len(self.shape) < 2:
+            return str(self)
 
         import numpy as np
         size_in_pixels = np.prod(self.shape)
@@ -195,6 +199,14 @@ def _imshow(image, title: str = None, labels: bool = False, min_display_intensit
     """
     import numpy as np
 
+    if len(image.shape) == 3 and image.shape[2] == 3: # RGB image
+        import matplotlib.pyplot as plt
+        plt.imshow(image, vmin=min_display_intensity, vmax=max_display_intensity,
+                   interpolation='nearest', alpha=alpha)
+        if not continue_drawing:
+            plt.show()
+        return
+
     if len(image.shape) == 3:
         image = np.asarray(image).max(axis=0)
 
